@@ -22,7 +22,8 @@ from pyiceberg.catalog import Catalog, load_catalog
 from pyiceberg.exceptions import CommitFailedException, NoSuchTableError, ValidationError
 from pyiceberg.partitioning import PartitionField, PartitionSpec
 from pyiceberg.schema import Schema, prune_columns
-from pyiceberg.table import Table, UpdateSchema
+from pyiceberg.table import Table, TableProperties, UpdateSchema
+from pyiceberg.table.name_mapping import MappedField, NameMapping, create_mapping_from_schema
 from pyiceberg.table.sorting import SortField, SortOrder
 from pyiceberg.transforms import IdentityTransform
 from pyiceberg.types import (
@@ -73,13 +74,17 @@ def _create_table_with_schema(catalog: Catalog, schema: Schema) -> Table:
         catalog.drop_table(tbl_name)
     except NoSuchTableError:
         pass
-    return catalog.create_table(identifier=tbl_name, schema=schema)
+    return catalog.create_table(
+        identifier=tbl_name,
+        schema=schema,
+        properties={TableProperties.DEFAULT_NAME_MAPPING: create_mapping_from_schema(schema).model_dump_json()},
+    )
 
 
 @pytest.mark.integration
 def test_add_already_exists(catalog: Catalog, table_schema_nested: Schema) -> None:
     table = _create_table_with_schema(catalog, table_schema_nested)
-    update = UpdateSchema(table)
+    update = table.update_schema()
 
     with pytest.raises(ValueError) as exc_info:
         update.add_column("foo", IntegerType())
@@ -93,7 +98,7 @@ def test_add_already_exists(catalog: Catalog, table_schema_nested: Schema) -> No
 @pytest.mark.integration
 def test_add_to_non_struct_type(catalog: Catalog, table_schema_simple: Schema) -> None:
     table = _create_table_with_schema(catalog, table_schema_simple)
-    update = UpdateSchema(table)
+    update = table.update_schema()
     with pytest.raises(ValueError) as exc_info:
         update.add_column(path=("foo", "lat"), field_type=IntegerType())
     assert "Cannot add column 'lat' to non-struct type: foo" in str(exc_info.value)
@@ -353,22 +358,22 @@ def test_revert_changes(simple_table: Table, table_schema_simple: Schema) -> Non
 
     assert simple_table.schemas() == {
         0: Schema(
-            NestedField(field_id=1, name='foo', field_type=StringType(), required=False),
-            NestedField(field_id=2, name='bar', field_type=IntegerType(), required=True),
-            NestedField(field_id=3, name='baz', field_type=BooleanType(), required=False),
-            schema_id=0,
+            NestedField(field_id=1, name="foo", field_type=StringType(), required=False),
+            NestedField(field_id=2, name="bar", field_type=IntegerType(), required=True),
+            NestedField(field_id=3, name="baz", field_type=BooleanType(), required=False),
             identifier_field_ids=[2],
         ),
         1: Schema(
-            NestedField(field_id=1, name='foo', field_type=StringType(), required=False),
-            NestedField(field_id=2, name='bar', field_type=IntegerType(), required=True),
-            NestedField(field_id=3, name='baz', field_type=BooleanType(), required=False),
-            NestedField(field_id=4, name='data', field_type=IntegerType(), required=False),
-            schema_id=1,
+            NestedField(field_id=1, name="foo", field_type=StringType(), required=False),
+            NestedField(field_id=2, name="bar", field_type=IntegerType(), required=True),
+            NestedField(field_id=3, name="baz", field_type=BooleanType(), required=False),
+            NestedField(field_id=4, name="data", field_type=IntegerType(), required=False),
             identifier_field_ids=[2],
         ),
     }
     assert simple_table.schema().schema_id == 0
+    assert simple_table.schemas()[0].schema_id == 0
+    assert simple_table.schemas()[1].schema_id == 1
 
 
 @pytest.mark.integration
@@ -667,13 +672,24 @@ def test_rename_simple(simple_table: Table) -> None:
     with simple_table.update_schema() as schema_update:
         schema_update.rename_column("foo", "vo")
 
+    with simple_table.transaction() as txn:
+        with txn.update_schema() as schema_update:
+            schema_update.rename_column("bar", "var")
+
     assert simple_table.schema() == Schema(
         NestedField(field_id=1, name="vo", field_type=StringType(), required=False),
-        NestedField(field_id=2, name="bar", field_type=IntegerType(), required=True),
+        NestedField(field_id=2, name="var", field_type=IntegerType(), required=True),
         NestedField(field_id=3, name="baz", field_type=BooleanType(), required=False),
         identifier_field_ids=[2],
     )
 
+    # Check that the name mapping gets updated
+    assert simple_table.name_mapping() == NameMapping([
+        MappedField(field_id=1, names=["foo", "vo"]),
+        MappedField(field_id=2, names=["bar", "var"]),
+        MappedField(field_id=3, names=["baz"]),
+    ])
+
 
 @pytest.mark.integration
 def test_rename_simple_nested(catalog: Catalog) -> None:
@@ -701,6 +717,11 @@ def test_rename_simple_nested(catalog: Catalog) -> None:
         ),
     )
 
+    # Check that the name mapping gets updated
+    assert tbl.name_mapping() == NameMapping([
+        MappedField(field_id=1, names=["foo"], fields=[MappedField(field_id=2, names=["bar", "vo"])]),
+    ])
+
 
 @pytest.mark.integration
 def test_rename_simple_nested_with_dots(catalog: Catalog) -> None:
@@ -957,7 +978,7 @@ def test_add_nested_map_of_structs(catalog: Catalog) -> None:
     tbl = _create_table_with_schema(
         catalog,
         Schema(
-            NestedField(field_id=1, name="foo", field_type=StringType()),
+            NestedField(field_id=1, name="foo", field_type=StringType(), required=True),
         ),
     )
 
@@ -1010,7 +1031,7 @@ def test_add_nested_list_of_structs(catalog: Catalog) -> None:
     tbl = _create_table_with_schema(
         catalog,
         Schema(
-            NestedField(field_id=1, name="foo", field_type=StringType()),
+            NestedField(field_id=1, name="foo", field_type=StringType(), required=True),
         ),
     )
 
@@ -1049,13 +1070,13 @@ def test_add_nested_list_of_structs(catalog: Catalog) -> None:
 def test_add_required_column(catalog: Catalog) -> None:
     schema_ = Schema(NestedField(field_id=1, name="a", field_type=BooleanType(), required=False))
     table = _create_table_with_schema(catalog, schema_)
-    update = UpdateSchema(table)
+    update = table.update_schema()
     with pytest.raises(ValueError) as exc_info:
         update.add_column(path="data", field_type=IntegerType(), required=True)
     assert "Incompatible change: cannot add required column: data" in str(exc_info.value)
 
     new_schema = (
-        UpdateSchema(table, allow_incompatible_changes=True)  # pylint: disable=W0212
+        UpdateSchema(transaction=table.transaction(), allow_incompatible_changes=True)
         .add_column(path="data", field_type=IntegerType(), required=True)
         ._apply()
     )
@@ -1071,12 +1092,13 @@ def test_add_required_column_case_insensitive(catalog: Catalog) -> None:
     table = _create_table_with_schema(catalog, schema_)
 
     with pytest.raises(ValueError) as exc_info:
-        with UpdateSchema(table, allow_incompatible_changes=True) as update:
-            update.case_sensitive(False).add_column(path="ID", field_type=IntegerType(), required=True)
+        with table.transaction() as txn:
+            with txn.update_schema(allow_incompatible_changes=True) as update:
+                update.case_sensitive(False).add_column(path="ID", field_type=IntegerType(), required=True)
     assert "already exists: ID" in str(exc_info.value)
 
     new_schema = (
-        UpdateSchema(table, allow_incompatible_changes=True)  # pylint: disable=W0212
+        UpdateSchema(transaction=table.transaction(), allow_incompatible_changes=True)
         .add_column(path="ID", field_type=IntegerType(), required=True)
         ._apply()
     )
@@ -1247,7 +1269,7 @@ def test_mixed_changes(catalog: Catalog) -> None:
 @pytest.mark.integration
 def test_ambiguous_column(catalog: Catalog, table_schema_nested: Schema) -> None:
     table = _create_table_with_schema(catalog, table_schema_nested)
-    update = UpdateSchema(table)
+    update = UpdateSchema(transaction=table.transaction())
 
     with pytest.raises(ValueError) as exc_info:
         update.add_column(path="location.latitude", field_type=IntegerType())
@@ -2490,15 +2512,17 @@ def test_two_add_schemas_in_a_single_transaction(catalog: Catalog) -> None:
         ),
     )
 
-    with pytest.raises(ValueError) as exc_info:
-        with tbl.transaction() as tr:
-            with tr.update_schema() as update:
-                update.add_column("bar", field_type=StringType())
-            with tr.update_schema() as update:
-                update.add_column("baz", field_type=StringType())
+    with tbl.transaction() as tr:
+        with tr.update_schema() as update:
+            update.add_column("bar", field_type=StringType())
+        with tr.update_schema() as update:
+            update.add_column("baz", field_type=StringType())
 
-    assert "Updates in a single commit need to be unique, duplicate: <class 'pyiceberg.table.AddSchemaUpdate'>" in str(
-        exc_info.value
+    assert tbl.schema().schema_id == 2
+    assert tbl.schema() == Schema(
+        NestedField(field_id=1, name="foo", field_type=StringType()),
+        NestedField(field_id=2, name="bar", field_type=StringType()),
+        NestedField(field_id=3, name="baz", field_type=StringType()),
     )
 
 
